@@ -6,8 +6,8 @@ import (
 	"os"
 	"testing"
 
+	"github.com/alecthomas/assert/v2"
 	"github.com/alecthomas/kong"
-	"github.com/stretchr/testify/require"
 )
 
 func TestMultipleConfigLoading(t *testing.T) {
@@ -25,13 +25,13 @@ func TestMultipleConfigLoading(t *testing.T) {
 
 	p := mustNew(t, &cli, kong.Configuration(kong.JSON, first, second))
 	_, err := p.Parse(nil)
-	require.NoError(t, err)
-	require.Equal(t, "first", cli.Flag)
+	assert.NoError(t, err)
+	assert.Equal(t, "first", cli.Flag)
 }
 
 func TestConfigValidation(t *testing.T) {
 	var cli struct {
-		Flag string `json:"flag,omitempty" enum:"valid"`
+		Flag string `json:"flag,omitempty" enum:"valid" required:""`
 	}
 
 	cli.Flag = "invalid"
@@ -40,15 +40,15 @@ func TestConfigValidation(t *testing.T) {
 
 	p := mustNew(t, &cli, kong.Configuration(kong.JSON, conf))
 	_, err := p.Parse(nil)
-	require.Error(t, err)
+	assert.Error(t, err)
 }
 
 func makeConfig(t *testing.T, config interface{}) (path string, cleanup func()) {
 	t.Helper()
 	w, err := ioutil.TempFile("", "")
-	require.NoError(t, err)
-	defer w.Close()
+	assert.NoError(t, err)
+	defer w.Close() // nolint: gosec
 	err = json.NewEncoder(w).Encode(config)
-	require.NoError(t, err)
+	assert.NoError(t, err)
 	return w.Name(), func() { os.Remove(w.Name()) }
 }
