@@ -6,11 +6,22 @@ import (
 	"strings"
 	"testing"
 
-	"github.com/stretchr/testify/require"
-
+	"github.com/alecthomas/assert/v2"
 	"github.com/alecthomas/kong"
 )
 
+func panicsTrue(t *testing.T, f func()) {
+	defer func() {
+		if value := recover(); value != nil {
+			if boolval, ok := value.(bool); !ok || !boolval {
+				t.Fatalf("expected panic with true but got %v", value)
+			}
+		}
+	}()
+	f()
+	t.Fatal("expected panic did not occur")
+}
+
 type threeArg struct {
 	RequiredThree bool   `required`
 	Three         string `arg`
@@ -35,11 +46,11 @@ func TestHelpOptionalArgs(t *testing.T) {
 			panic(true) // Panic to fake "exit".
 		}),
 	)
-	require.PanicsWithValue(t, true, func() {
+	panicsTrue(t, func() {
 		_, err := app.Parse([]string{"--help"})
-		require.NoError(t, err)
+		assert.NoError(t, err)
 	})
-	require.True(t, exited)
+	assert.True(t, exited)
 	expected := `Usage: test-app [<one> [<two>]]
 
 Arguments:
@@ -49,7 +60,7 @@ Arguments:
 Flags:
   -h, --help    Show context-sensitive help.
 `
-	require.Equal(t, expected, w.String())
+	assert.Equal(t, expected, w.String())
 }
 
 func TestHelp(t *testing.T) {
@@ -89,11 +100,11 @@ func TestHelp(t *testing.T) {
 	)
 
 	t.Run("Full", func(t *testing.T) {
-		require.PanicsWithValue(t, true, func() {
+		panicsTrue(t, func() {
 			_, err := app.Parse([]string{"--help"})
-			require.NoError(t, err)
+			assert.NoError(t, err)
 		})
-		require.True(t, exited)
+		assert.True(t, exited)
 		expected := `Usage: test-app --required <command>
 
 A test app.
@@ -122,17 +133,17 @@ Run "test-app <command> --help" for more information on a command.
 `
 		t.Log(w.String())
 		t.Log(expected)
-		require.Equal(t, expected, w.String())
+		assert.Equal(t, expected, w.String())
 	})
 
 	t.Run("Selected", func(t *testing.T) {
 		exited = false
 		w.Truncate(0)
-		require.PanicsWithValue(t, true, func() {
+		panicsTrue(t, func() {
 			_, err := app.Parse([]string{"two", "hello", "--help"})
-			require.NoError(t, err)
+			assert.NoError(t, err)
 		})
-		require.True(t, exited)
+		assert.True(t, exited)
 		expected := `Usage: test-app two <three> --required --required-two --required-three
 
 Sub-sub-arg.
@@ -156,7 +167,7 @@ Flags:
 `
 		t.Log(expected)
 		t.Log(w.String())
-		require.Equal(t, expected, w.String())
+		assert.Equal(t, expected, w.String())
 	})
 }
 
@@ -199,11 +210,11 @@ func TestFlagsLast(t *testing.T) {
 	)
 
 	t.Run("Full", func(t *testing.T) {
-		require.PanicsWithValue(t, true, func() {
+		panicsTrue(t, func() {
 			_, err := app.Parse([]string{"--help"})
-			require.NoError(t, err)
+			assert.NoError(t, err)
 		})
-		require.True(t, exited)
+		assert.True(t, exited)
 		expected := `Usage: test-app --required <command>
 
 A test app.
@@ -231,17 +242,17 @@ Run "test-app <command> --help" for more information on a command.
 `
 		t.Log(w.String())
 		t.Log(expected)
-		require.Equal(t, expected, w.String())
+		assert.Equal(t, expected, w.String())
 	})
 
 	t.Run("Selected", func(t *testing.T) {
 		exited = false
 		w.Truncate(0)
-		require.PanicsWithValue(t, true, func() {
+		panicsTrue(t, func() {
 			_, err := app.Parse([]string{"two", "hello", "--help"})
-			require.NoError(t, err)
+			assert.NoError(t, err)
 		})
-		require.True(t, exited)
+		assert.True(t, exited)
 		expected := `Usage: test-app two <three> --required --required-two --required-three
 
 Sub-sub-arg.
@@ -264,7 +275,7 @@ Flags:
 `
 		t.Log(expected)
 		t.Log(w.String())
-		require.Equal(t, expected, w.String())
+		assert.Equal(t, expected, w.String())
 	})
 }
 
@@ -304,11 +315,11 @@ func TestHelpTree(t *testing.T) {
 	)
 
 	t.Run("Full", func(t *testing.T) {
-		require.PanicsWithValue(t, true, func() {
+		panicsTrue(t, func() {
 			_, err := app.Parse([]string{"--help"})
-			require.NoError(t, err)
+			assert.NoError(t, err)
 		})
-		require.True(t, exited)
+		assert.True(t, exited)
 		expected := `Usage: test-app <command>
 
 A test app.
@@ -331,17 +342,17 @@ Run "test-app <command> --help" for more information on a command.
 		if expected != w.String() {
 			t.Errorf("help command returned:\n%v\n\nwant:\n%v", w.String(), expected)
 		}
-		require.Equal(t, expected, w.String())
+		assert.Equal(t, expected, w.String())
 	})
 
 	t.Run("Selected", func(t *testing.T) {
 		exited = false
 		w.Truncate(0)
-		require.PanicsWithValue(t, true, func() {
+		panicsTrue(t, func() {
 			_, err := app.Parse([]string{"one", "--help"})
-			require.NoError(t, err)
+			assert.NoError(t, err)
 		})
-		require.True(t, exited)
+		assert.True(t, exited)
 		expected := `Usage: test-app one (un,uno) <command>
 
 subcommand one
@@ -358,7 +369,7 @@ Commands:
 		if expected != w.String() {
 			t.Errorf("help command returned:\n%v\n\nwant:\n%v", w.String(), expected)
 		}
-		require.Equal(t, expected, w.String())
+		assert.Equal(t, expected, w.String())
 	})
 }
 
@@ -398,11 +409,11 @@ func TestHelpCompactNoExpand(t *testing.T) {
 	)
 
 	t.Run("Full", func(t *testing.T) {
-		require.PanicsWithValue(t, true, func() {
+		panicsTrue(t, func() {
 			_, err := app.Parse([]string{"--help"})
-			require.NoError(t, err)
+			assert.NoError(t, err)
 		})
-		require.True(t, exited)
+		assert.True(t, exited)
 		expected := `Usage: test-app <command>
 
 A test app.
@@ -421,17 +432,17 @@ Run "test-app <command> --help" for more information on a command.
 		if expected != w.String() {
 			t.Errorf("help command returned:\n%v\n\nwant:\n%v", w.String(), expected)
 		}
-		require.Equal(t, expected, w.String())
+		assert.Equal(t, expected, w.String())
 	})
 
 	t.Run("Selected", func(t *testing.T) {
 		exited = false
 		w.Truncate(0)
-		require.PanicsWithValue(t, true, func() {
+		panicsTrue(t, func() {
 			_, err := app.Parse([]string{"one", "--help"})
-			require.NoError(t, err)
+			assert.NoError(t, err)
 		})
-		require.True(t, exited)
+		assert.True(t, exited)
 		expected := `Usage: test-app one (un,uno) <command>
 
 subcommand one
@@ -446,7 +457,7 @@ Group A
 		if expected != w.String() {
 			t.Errorf("help command returned:\n%v\n\nwant:\n%v", w.String(), expected)
 		}
-		require.Equal(t, expected, w.String())
+		assert.Equal(t, expected, w.String())
 	})
 }
 
@@ -457,8 +468,8 @@ func TestEnvarAutoHelp(t *testing.T) {
 	w := &strings.Builder{}
 	p := mustNew(t, &cli, kong.Writers(w, w), kong.Exit(func(int) {}))
 	_, err := p.Parse([]string{"--help"})
-	require.NoError(t, err)
-	require.Contains(t, w.String(), "A flag ($FLAG).")
+	assert.NoError(t, err)
+	assert.Contains(t, w.String(), "A flag ($FLAG).")
 }
 
 func TestEnvarAutoHelpWithEnvPrefix(t *testing.T) {
@@ -472,9 +483,9 @@ func TestEnvarAutoHelpWithEnvPrefix(t *testing.T) {
 	w := &strings.Builder{}
 	p := mustNew(t, &cli, kong.Writers(w, w), kong.Exit(func(int) {}))
 	_, err := p.Parse([]string{"--help"})
-	require.NoError(t, err)
-	require.Contains(t, w.String(), "A flag ($ANON_FLAG).")
-	require.Contains(t, w.String(), "A different flag.")
+	assert.NoError(t, err)
+	assert.Contains(t, w.String(), "A flag ($ANON_FLAG).")
+	assert.Contains(t, w.String(), "A different flag.")
 }
 
 func TestCustomValueFormatter(t *testing.T) {
@@ -490,8 +501,83 @@ func TestCustomValueFormatter(t *testing.T) {
 		}),
 	)
 	_, err := p.Parse([]string{"--help"})
-	require.NoError(t, err)
-	require.Contains(t, w.String(), "A flag.")
+	assert.NoError(t, err)
+	assert.Contains(t, w.String(), "A flag.")
+}
+
+func TestAutoGroup(t *testing.T) {
+	var cli struct {
+		GroupedAString string `help:"A string flag grouped in A."`
+		FreeString     string `help:"A non grouped string flag."`
+		GroupedBString string `help:"A string flag grouped in B."`
+		FreeBool       bool   `help:"A non grouped bool flag."`
+		GroupedABool   bool   `help:"A bool flag grouped in A."`
+
+		One struct {
+			Flag string `help:"Nested flag."`
+			// Group is inherited from the parent command
+			Thing struct {
+				Arg string `arg help:"argument"`
+			} `cmd help:"subcommand thing"`
+			Other struct {
+				Other string `arg help:"other arg"`
+			} `arg help:"subcommand other"`
+			// ... but a subcommand can override it
+			Stuff struct {
+				Stuff string `arg help:"argument"`
+			} `arg help:"subcommand stuff"`
+		} `cmd help:"A subcommand grouped in A."`
+
+		Two struct {
+			Grouped1String  string `help:"A string flag grouped in 1."`
+			AFreeString     string `help:"A non grouped string flag."`
+			Grouped2String  string `help:"A string flag grouped in 2."`
+			AGroupedAString bool   `help:"A string flag grouped in A."`
+			Grouped1Bool    bool   `help:"A bool flag grouped in 1."`
+		} `cmd help:"A non grouped subcommand."`
+
+		Four struct {
+			Flag string `help:"Nested flag."`
+		} `cmd help:"Another subcommand grouped in B."`
+
+		Three struct {
+			Flag string `help:"Nested flag."`
+		} `cmd help:"Another subcommand grouped in A."`
+	}
+	w := bytes.NewBuffer(nil)
+	app := mustNew(t, &cli,
+		kong.Writers(w, w),
+		kong.Exit(func(int) {}),
+		kong.AutoGroup(func(parent kong.Visitable, flag *kong.Flag) *kong.Group {
+			if node, ok := parent.(*kong.Node); ok {
+				return &kong.Group{
+					Key:   node.Name,
+					Title: strings.Title(node.Name) + " flags:", // nolint
+				}
+			}
+			return nil
+		}),
+	)
+	_, _ = app.Parse([]string{"--help", "two"})
+	assert.Equal(t, `Usage: test two
+
+A non grouped subcommand.
+
+Flags:
+  -h, --help                       Show context-sensitive help.
+      --grouped-a-string=STRING    A string flag grouped in A.
+      --free-string=STRING         A non grouped string flag.
+      --grouped-b-string=STRING    A string flag grouped in B.
+      --free-bool                  A non grouped bool flag.
+      --grouped-a-bool             A bool flag grouped in A.
+
+Two flags:
+  --grouped-1-string=STRING    A string flag grouped in 1.
+  --a-free-string=STRING       A non grouped string flag.
+  --grouped-2-string=STRING    A string flag grouped in 2.
+  --a-grouped-a-string         A string flag grouped in A.
+  --grouped-1-bool             A bool flag grouped in 1.
+`, w.String())
 }
 
 func TestHelpGrouping(t *testing.T) {
@@ -552,10 +638,10 @@ func TestHelpGrouping(t *testing.T) {
 	)
 
 	t.Run("Full", func(t *testing.T) {
-		require.PanicsWithValue(t, true, func() {
+		panicsTrue(t, func() {
 			_, err := app.Parse([]string{"--help"})
-			require.True(t, exited)
-			require.NoError(t, err)
+			assert.True(t, exited)
+			assert.NoError(t, err)
 		})
 		expected := `Usage: test-app <command>
 
@@ -602,16 +688,16 @@ Run "test-app <command> --help" for more information on a command.
 `
 		t.Log(w.String())
 		t.Log(expected)
-		require.Equal(t, expected, w.String())
+		assert.Equal(t, expected, w.String())
 	})
 
 	t.Run("Selected", func(t *testing.T) {
 		exited = false
 		w.Truncate(0)
-		require.PanicsWithValue(t, true, func() {
+		panicsTrue(t, func() {
 			_, err := app.Parse([]string{"two", "--help"})
-			require.NoError(t, err)
-			require.True(t, exited)
+			assert.NoError(t, err)
+			assert.True(t, exited)
 		})
 		expected := `Usage: test-app two
 
@@ -644,7 +730,7 @@ Group 2
 `
 		t.Log(expected)
 		t.Log(w.String())
-		require.Equal(t, expected, w.String())
+		assert.Equal(t, expected, w.String())
 	})
 }
 
@@ -672,7 +758,7 @@ Flags:
 
 test: error: missing flags: --flag=STRING
 `
-	require.Equal(t, expected, w.String())
+	assert.Equal(t, expected, w.String())
 }
 
 func TestShortUsageOnError(t *testing.T) {
@@ -687,7 +773,7 @@ func TestShortUsageOnError(t *testing.T) {
 		kong.ShortUsageOnError(),
 	)
 	_, err := p.Parse([]string{})
-	require.Error(t, err)
+	assert.Error(t, err)
 	p.FatalIfErrorf(err)
 
 	expected := `Usage: test --flag=STRING
@@ -695,7 +781,7 @@ Run "test --help" for more information.
 
 test: error: missing flags: --flag=STRING
 `
-	require.Equal(t, expected, w.String())
+	assert.Equal(t, expected, w.String())
 }
 
 func TestCustomShortUsageOnError(t *testing.T) {
@@ -715,47 +801,12 @@ func TestCustomShortUsageOnError(t *testing.T) {
 		kong.ShortUsageOnError(),
 	)
 	_, err := p.Parse([]string{})
-	require.Error(t, err)
+	assert.Error(t, err)
 	p.FatalIfErrorf(err)
 
 	expected := `🤷 wish I could help
 
 test: error: missing flags: --flag=STRING
 `
-	require.Equal(t, expected, w.String())
-}
-
-func TestCustomWrap(t *testing.T) {
-	var cli struct {
-		Flag string `help:"A string flag with very long help that wraps a lot and is verbose and is really verbose."`
-	}
-
-	w := bytes.NewBuffer(nil)
-	app := mustNew(t, &cli,
-		kong.Name("test-app"),
-		kong.Description("A test app."),
-		kong.HelpOptions{
-			WrapUpperBound: 50,
-		},
-		kong.Writers(w, w),
-		kong.Exit(func(int) {}),
-	)
-
-	_, err := app.Parse([]string{"--help"})
-	require.NoError(t, err)
-	expected := `Usage: test-app
-
-A test app.
-
-Flags:
-  -h, --help           Show context-sensitive
-                       help.
-      --flag=STRING    A string flag with very
-                       long help that wraps a lot
-                       and is verbose and is
-                       really verbose.
-`
-	t.Log(w.String())
-	t.Log(expected)
-	require.Equal(t, expected, w.String())
+	assert.Equal(t, expected, w.String())
 }
